/*****************************************************************************
 *                                  _   _ ____  _     
 *  Project                     ___| | | |  _ \| |    
 *                             / __| | | | |_) | |    
 *                            | (__| |_| |  _ <| |___ 
 *                             \___|\___/|_| \_\_____|
 *
 * Copyright (C) 2001, Daniel Stenberg, <daniel@haxx.se>, et al.
 *
 * In order to be useful for every potential user, curl and libcurl are
 * dual-licensed under the MPL and the MIT/X-derivate licenses.
 *
 * You may opt to use, copy, modify, merge, publish, distribute and/or sell
 * copies of the Software, and permit persons to whom the Software is
 * furnished to do so, under the terms of the MPL or the MIT/X-derivate
 * licenses. You may pick one of these licenses.
 *
 * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
 * KIND, either express or implied.
 *
 *****************************************************************************/

#include "setup.h"

#include <stdlib.h>

#include "export_to_user.h"
#include "sync.h"

/* The last #include file should be: */
#ifdef MALLOCDEBUG
#  include "memdebug.h"
#endif

/*
 * The one and only global pointer to the mutex functions that is used within libcURL
 */
static curl_mutex_functions_t* global_mutex_functions = NULL;


/*
 * Register a mutex function struct
 */
void curl_register_mutex_functions(curl_mutex_functions_t* mf)
{
  global_mutex_functions= mf;
}

/*
 * Unregister it
 */
void curl_unregister_mutex_functions(void)
{
  global_mutex_functions= NULL;
}

/*
 * Get the pointer to the global mutex function struct
 */
curl_mutex_functions_t* curl_get_mutex_functions(void)
{
  return (global_mutex_functions);
}


/*
 * The convenience funtions for libcURL to operate on the user mutexes.
 */
curl_mutex_t* Curl_mutex_new(void)
{
  if (global_mutex_functions == NULL)
    return NULL;
  else
    return global_mutex_functions->new_function();
}

void Curl_mutex_free(curl_mutex_t* m)
{
  if (global_mutex_functions != NULL)
    global_mutex_functions->free_function(m);
}

void Curl_mutex_lock(curl_mutex_t* m)
{
  if (global_mutex_functions != NULL)
    global_mutex_functions->lock_function(m);
}

void Curl_mutex_unlock(curl_mutex_t* m)
{
  if (global_mutex_functions != NULL)
    global_mutex_functions->unlock_function(m);
}
